--==============================================================================
-- Copyright (c) 2005 by STMicroelectronics.  All rights reserved.
--
--   Filename    : vr_ad_axi_config.e
--
--   Purpose     : vr_ad/axi eVC integration
--
--   Limitations : none
--
--   Authors     : Olivier Schuler (TINA 041-4030)
--                 
--   Last update : 2006-03-01
--
--   Tools       : Specman Elite 4.3.4
--
--==============================================================================
<'

--Import the vr_ad package done in dynamic library
--import vr_ad/e/vr_ad_top;
    
//////////////////////////////////////////////////////////////////////
// 1. Connect the eVC to the address map of the register model
//////////////////////////////////////////////////////////////////////
    
extend IF0 ACTIVE vr_axi_master {
   addr_map : vr_ad_map;
   reg_driver : vr_ad_sequence_driver is instance;
      keep reg_driver.addr_map == value(addr_map);
      keep reg_driver.default_bfm_sd == driver;
   reg_file : REG_FILE vr_ad_reg_file;

   post_generate() is also {
      addr_map.add_with_offset(0x0,reg_file);
   }; -- post_generate()
};

//////////////////////////////////////////////////////////////////////
// 2. Implement the vr_ad_execute_op() method for your eVC SD (Sequence Driver)
//    -> Only required when the vr_ad package is used for driving registers.
//////////////////////////////////////////////////////////////////////

extend vr_axi_master_driver {
   rdata : list of vr_axi_memory_entry;
      keep rdata.size()==0;
   vr_ad_execute_op(operation : vr_ad_operation) : list of byte @clock is {
      if operation is a REG vr_ad_operation (op) {
         var i : uint;
         var lod   : list of vr_axi_data_t;
         var rsize : vr_axi_transfer_size_t;
         var bkind : vr_axi_burst_kind_t; 
         var write_response : vr_axi_response_t;
         var read_data : list of vr_axi_memory_entry;
         
         rsize = ilog2(op.reg.get_size()).as_a(vr_axi_transfer_size_t);
         bkind = FIXED;
            
         if operation.direction == WRITE { 
            // Write a single register
            lod.add(op.reg.read_reg_val().as_a(vr_axi_data_t));
            write_response = sequence.write_burst(op.address, 1, rsize, bkind, lod);
         } else { 
            for {i=0;i<op.reg.get_size();i+=1} {
               result.add(0x0);
            };
            rdata = sequence.read_burst(op.address, 1, rsize, bkind);
            for each (e) in rdata {
               result.add(e.data);
            };
         };    
      };
   }; 
};


//////////////////////////////////////////////////////////////////////
// 3. Extending the monitor to update the register model.
//////////////////////////////////////////////////////////////////////
    
extend AXI_0 IF0 MASTER vr_axi_agent_monitor {
   
   on write_burst_ended {
      if (cur_write_transfer.p_burst.as_a(WRITE'direction vr_axi_monitor_burst).resp == OKAY) {
         var current_data : list of byte = get_data_from_transfer(cur_write_transfer);
         get_enclosing_unit(vr_axi_master).as_a(IF0'name ACTIVE'active_passive vr_axi_master).addr_map.update(cur_write_transfer.address, current_data,{});
      };
   };
   
   on read_burst_ended {
      if (cur_read_transfer.resp == OKAY) {
         var current_data : list of byte = get_data_from_transfer(cur_read_transfer);
         compute get_enclosing_unit(vr_axi_master).as_a(IF0'name ACTIVE'active_passive vr_axi_master).addr_map.compare_and_update(cur_read_transfer.address, current_data);
      };
   };
   
   get_data_from_transfer(c_trans: vr_axi_monitor_transfer) : list of byte is {
      var num_of_bytes : uint = (1<<c_trans.p_burst.size.as_a(uint));
      for i from 0 to (num_of_bytes - 1) {
         result.add(c_trans.data[((i+1) * 8) -1:i*8].as_a(byte));
      };
   };
};
 
   
//////////////////////////////////////////////////////////////////////
// 4. Define registers and registerfile (should be done in a separated file)
//////////////////////////////////////////////////////////////////////   
extend vr_ad_reg_file_kind : [REG_FILE];
extend REG_FILE vr_ad_reg_file {
    keep size == 8;
};

reg_def REG1 REG_FILE 0x00000000 {
   reg_fld data1 : uint(bits:32) : RW : 0x00000000 : cov;

   set_static_info() is {
      static_info.compare_mask = 0xffffffff;
   };
};

reg_def REG2 REG_FILE 0x00000004 {
   reg_fld cmd1 : uint(bits:32) : RW : 0x00000000 : cov;
   
   set_static_info() is {
      static_info.compare_mask = 0xffffffff;
   };
};

//////////////////////////////////////////////////////////////////////
// 5. Connect to the virtual sequence driver if any (should be done in a separated file)
//////////////////////////////////////////////////////////////////////  
extend vr_axi_virtual_seq_driver {
   master_reg_driver : vr_ad_sequence_driver;
};

extend AXI_0 vr_axi_env {
   keep virtual_seq_driver.master_reg_driver == masters[0].as_a(IF0 ACTIVE vr_axi_master).reg_driver;
};

//////////////////////////////////////////////////////////////////////
// 6. Define register subsequence library (should be done in a separated file)
//////////////////////////////////////////////////////////////////////  
extend vr_ad_sequence_kind : [REG_SEQ];

extend REG_SEQ vr_ad_sequence {
   !reg1 : REG1 vr_ad_reg;
   !reg2 : REG2 vr_ad_reg;

   body() @driver.clock is only {
      write_reg reg1;
      read_reg reg1;
      write_reg reg2;
      read_reg reg2;
   };
};


'>
